import gzip
import json
import pickle

import yaml


def load_traj_file(path: str):
    if path.endswith(".pkl"):
        with open(path, "rb") as f:
            return pickle.load(f)
    elif path.endswith(".pkl.gz"):
        with gzip.open(path, "rb") as f:
            return pickle.load(f)
    elif path.endswith(".json"):
        with open(path) as f:
            return json.load(f)
    elif path.endswith(".yaml"):
        with open(path) as f:
            return yaml.load(f, Loader=yaml.FullLoader)
    else:
        raise ValueError(f"Unsupported file extension: {path}")


def save_traj_file(data: dict, path: str):
    if path.endswith(".pkl"):
        with open(path, "wb") as f:
            pickle.dump(data, f)
    elif path.endswith(".pkl.gz"):
        with gzip.open(path, "wb", compresslevel=1) as f:
            pickle.dump(data, f)
    elif path.endswith(".json"):
        with open(path, "w") as f:
            json.dump(data, f)
    elif path.endswith(".yaml"):
        with open(path, "w") as f:
            yaml.dump(data, f)
    else:
        raise ValueError(f"Unsupported file extension: {path}")



# ruff: noqa: F401

from .square_d0_metacfg import SquareD0MetaCfg
from .square_d1_metacfg import SquareD1MetaCfg
from .square_d2_metacfg import SquareD2MetaCfg
from .stack_d0_metacfg import StackD0MetaCfg



from __future__ import annotations

from metasim.sim.base import BaseSimHandler, EnvWrapper
from metasim.types import Action, EnvState, Extra, Obs, Reward, Success, TimeOut


class DoubleSimEnv(EnvWrapper[BaseSimHandler]):
    def __init__(self, env_physics: type[EnvWrapper[BaseSimHandler]], env_render: type[EnvWrapper[BaseSimHandler]]):
        self.sim_env1 = env_physics  # physics
        self.sim_env2 = env_render  # render

    def reset(self, states: list[EnvState] | None = None):
        self.sim_env1.reset(states=states)
        return self.sim_env2.reset(states=states)

    def step(self, action: list[Action]) -> tuple[Obs, Reward, Success, TimeOut, Extra]:
        obs, reward, success, time_out, extra = self.sim_env1.step(action)
        states = self.sim_env1.handler.get_states()
        self.sim_env2.handler.set_states(states)
        self.sim_env2.handler.refresh_render()
        obs = self.sim_env2.handler.get_observation()
        return obs, reward, success, time_out, extra

    def render(self):
        self.sim_env2.render()

    def close(self):
        self.sim_env1.close()
        self.sim_env2.close()

    @property
    def handler(self) -> BaseSimHandler:
        # XXX: is it ok to return the physics handler?
        return self.sim_env1.handler



from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class PutPlateInColoredDishRackMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/put_plate_in_colored_dish_rack/v2"
    objects = [
        RigidObjMetaCfg(
            name="plate_visual",
            filepath="roboverse_data/assets/rlbench/put_plate_in_colored_dish_rack/plate_visual/usd/plate_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="dish_rack",
            filepath="roboverse_data/assets/rlbench/put_plate_in_colored_dish_rack/dish_rack/usd/dish_rack.usd",
            physics=PhysicStateType.GEOM,
        ),
        RigidObjMetaCfg(
            name="plate_stand",
            filepath="roboverse_data/assets/rlbench/put_plate_in_colored_dish_rack/plate_stand/usd/plate_stand.usd",
            physics=PhysicStateType.GEOM,
        ),
    ]
    # TODO: add checker



from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class MeatOnGrillMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/meat_on_grill/v2"
    objects = [
        RigidObjMetaCfg(
            name="grill_visual",
            filepath="roboverse_data/assets/rlbench/meat_off_grill/grill_visual/usd/grill_visual.usd",  # reuse meat_on_grill asset
            physics=PhysicStateType.GEOM,
        ),
        RigidObjMetaCfg(
            name="chicken_visual",
            filepath="roboverse_data/assets/rlbench/meat_off_grill/chicken_visual/usd/chicken_visual.usd",  # reuse meat_on_grill asset
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="steak_visual",
            filepath="roboverse_data/assets/rlbench/meat_off_grill/steak_visual/usd/steak_visual.usd",  # reuse meat_on_grill asset
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    # TODO: add checker



from __future__ import annotations

import math
from dataclasses import MISSING

from metasim.constants import PhysicStateType
from metasim.utils import configclass


@configclass
class BaseObjMetaCfg:
    name: str = MISSING
    fix_base_link: bool = False
    scale: float = 1.0


# File-based object
@configclass
class RigidObjMetaCfg(BaseObjMetaCfg):
    filepath: str | None = None
    urdf_path: str | None = None
    mjcf_path: str | None = None
    mesh_path: str | None = None
    physics: PhysicStateType = MISSING


@configclass
class NonConvexRigidObjMetaCfg(RigidObjMetaCfg):
    mesh_pose: list[float] = MISSING


@configclass
class ArticulationObjMetaCfg(BaseObjMetaCfg):
    filepath: str | None = None
    urdf_path: str | None = None
    mjcf_path: str | None = None


# Primitive object are all rigid objects
@configclass
class PrimitiveCubeMetaCfg(RigidObjMetaCfg):
    mass: float = 0.1
    color: list[float] = MISSING
    size: list[float] = MISSING
    physics: PhysicStateType = MISSING
    mjcf_path: str | None = None  # TODO: remove this field

    @property
    def half_size(self) -> list[float]:
        """
        For SAPIEN usage
        """
        return [size / 2 for size in self.size]

    @property
    def density(self) -> float:
        """
        For SAPIEN usage
        """
        return self.mass / (self.size[0] * self.size[1] * self.size[2])


@configclass
class PrimitiveSphereMetaCfg(RigidObjMetaCfg):
    mass: float = 0.1
    color: list[float] = MISSING
    radius: float = MISSING
    physics: PhysicStateType = MISSING

    @property
    def density(self) -> float:
        """
        For SAPIEN usage
        """
        return self.mass / (4 / 3 * math.pi * self.radius**3)
